@@ -5,8 +5,8 @@ import (
 	"net/http"
 	"time"
 
-	"github.com/AdguardTeam/dnsproxy/proxyutil"
 	"github.com/AdguardTeam/dnsproxy/upstream"
+	"github.com/AdguardTeam/golibs/mathutil"
 	"github.com/ameshkov/dnscrypt/v2"
 	"github.com/miekg/dns"
 	"github.com/quic-go/quic-go"
@@ -114,11 +114,26 @@ func (dctx *DNSContext) scrub() {
 		dctx.Res.SetEdns0(dctx.udpSize, dctx.doBit)
 	}
 
-	dctx.Res.Truncate(proxyutil.DNSSize(dctx.Proto == ProtoUDP, dctx.Req))
+	dctx.Res.Truncate(dnsSize(dctx.Proto == ProtoUDP, dctx.Req))
 	// Some devices require DNS message compression.
 	dctx.Res.Compress = true
 }
 
+// dnsSize returns the buffer size advertised in the requests OPT record.  When
+// the request is over TCP, it returns the maximum allowed size of 64KiB.
+func dnsSize(isUDP bool, r *dns.Msg) (size int) {
+	if !isUDP {
+		return dns.MaxMsgSize
+	}
+
+	var size16 uint16
+	if o := r.IsEdns0(); o != nil {
+		size16 = o.UDPSize()
+	}
+
+	return int(mathutil.Max(dns.MinMsgSize, size16))
+}
+
 // DoQVersion is an enumeration with supported DoQ versions.
 type DoQVersion int
 
