@@ -3,6 +3,7 @@
 package upstream
 
 import (
+	"context"
 	"crypto/tls"
 	"crypto/x509"
 	"fmt"
@@ -21,6 +22,7 @@ import (
 	"github.com/ameshkov/dnscrypt/v2"
 	"github.com/ameshkov/dnsstamps"
 	"github.com/miekg/dns"
+	"github.com/quic-go/quic-go"
 	"github.com/quic-go/quic-go/logging"
 )
 
@@ -38,6 +40,14 @@ type Upstream interface {
 	io.Closer
 }
 
+// QUICTraceFunc is a function that returns a [logging.ConnectionTracer]
+// specific for a given role and connection ID.
+type QUICTraceFunc func(
+	ctx context.Context,
+	role logging.Perspective,
+	connID quic.ConnectionID,
+) (tracer logging.ConnectionTracer)
+
 // Options for AddressToUpstream func.  With these options we can configure the
 // upstream properties.
 type Options struct {
@@ -72,9 +82,9 @@ type Options struct {
 	// Upstream.Exchange method returns any error caused by it.
 	VerifyDNSCryptCertificate func(cert *dnscrypt.Cert) error
 
-	// QUICTracer is an optional object that allows tracing every QUIC
+	// QUICTracer is an optional callback that allows tracing every QUIC
 	// connection and logging every packet that goes through.
-	QUICTracer logging.Tracer
+	QUICTracer QUICTraceFunc
 
 	// InsecureSkipVerify disables verifying the server's certificate.
 	InsecureSkipVerify bool
@@ -268,24 +278,26 @@ func addPort(u *url.URL, port int) {
 	}
 }
 
-// Write to log DNS request information that we are going to send
-func logBegin(upstreamAddress string, req *dns.Msg) {
+// logBegin logs the start of DNS request resolution.  It should be called right
+// before dialing the connection to the upstream.  n is the [network] that will
+// be used to send the request.
+func logBegin(upstreamAddress string, n network, req *dns.Msg) {
 	qtype := ""
 	target := ""
 	if len(req.Question) != 0 {
 		qtype = dns.Type(req.Question[0].Qtype).String()
 		target = req.Question[0].Name
 	}
-	log.Debug("%s: sending request %s %s", upstreamAddress, qtype, target)
+	log.Debug("%s: sending request over %s: %s %s", upstreamAddress, n, qtype, target)
 }
 
 // Write to log about the result of DNS request
-func logFinish(upstreamAddress string, err error) {
+func logFinish(upstreamAddress string, n network, err error) {
 	status := "ok"
 	if err != nil {
 		status = err.Error()
 	}
-	log.Debug("%s: response: %s", upstreamAddress, status)
+	log.Debug("%s: response received over %s: %s", upstreamAddress, n, status)
 }
 
 // DialerInitializer returns the handler that it creates.  All the subsequent
